@@ -4,20 +4,37 @@ use Mojo::Base -strict, -signatures;
 
 use BoardStreams::Registry;
 
-use Mojo::JSON 'to_json';
+use Mojo::JSON 'to_json', 'decode_json';
 use RxPerl::Mojo ':all';
 
 use Exporter 'import';
 our @EXPORT_OK = qw/ get_listener_observable /;
 
-our $VERSION = "v0.0.5";
+our $VERSION = "v0.0.6";
 
 my %listener_observables;
 
 sub get_listener_observable ($c, $channel_name, $prefix) {
     return $listener_observables{$channel_name} //= rx_observable->new(sub ($subscriber) {
-        my $cb = $c->$prefix->pubsub->json($channel_name)->listen($channel_name => sub ($pubsub, $payload) {
-            $subscriber->next($payload);
+        my $accumulated;
+        my $cb = $c->$prefix->pubsub->listen($channel_name => sub ($pubsub, $payload) {
+            $payload =~ s/^\:([^:]+)\: //;
+            if (defined(my $prefix = $1)) {
+                if ($prefix eq '0') {
+                    $accumulated = $payload;
+                } elsif ($prefix =~ /^\d+\z/) {
+                    $accumulated .= $payload;
+                } elsif ($prefix eq 'end') {
+                    $accumulated .= $payload;
+                    # next accepts character hashrefs
+                    $subscriber->next(decode_json $accumulated);
+                    undef $accumulated;
+                }
+            } else {
+                # next accepts character hashrefs
+                $subscriber->next(decode_json $payload);
+                undef $accumulated;
+            }
         });
 
         return sub {
