@@ -26,11 +26,14 @@ use Carp 'croak';
 
 use experimental 'postderef';
 
-our $VERSION = "v0.0.5";
+our $VERSION = "v0.0.6";
 
 use constant {
-    DEFAULT_HEARTBEAT_INTERVAL => 5,
-    DEFAULT_HEARTBEAT_TIMEOUT  => 10,
+    DEFAULT_HEARTBEAT_INTERVAL       => 5,
+    DEFAULT_HEARTBEAT_TIMEOUT        => 10,
+    DEFAULT_PREFIX                   => 'bs',
+    DEFAULT_PING_INTERVAL            => 15,
+    DEFAULT_NOTIFY_PALOAD_SIZE_LIMIT => 8000,
 };
 
 my $WORKERS_CHANNEL = '_bs:workers';
@@ -41,7 +44,9 @@ sub register ($self, $app, $config) {
     # config params
     my $heartbeat_interval = $config->{heartbeat_interval} // DEFAULT_HEARTBEAT_INTERVAL;
     my $heartbeat_timeout = $config->{heartbeat_timeout} // DEFAULT_HEARTBEAT_TIMEOUT;
-    my $prefix = $config->{prefix} // 'bs';
+    my $prefix = $config->{prefix} // DEFAULT_PREFIX;
+    my $ping_interval = $config->{ping_interval} // DEFAULT_PING_INTERVAL;
+    my $notify_payload_size_limit = ($config->{notify_payload_size_limit} // DEFAULT_NOTIFY_PALOAD_SIZE_LIMIT) - 1;
 
     # Database stuff
     my $db_string = $config->{'db_string'} or die "missing db_string configuration option";
@@ -65,7 +70,15 @@ sub register ($self, $app, $config) {
     }
 
     # on workers startup:
+    my $ping_o;
     Mojo::IOLoop->next_tick(sub {
+        # ping every 15 seconds
+        $ping_o = rx_defer(sub {
+            rx_timer(rand($ping_interval), $ping_interval);
+        })->pipe(
+            op_share(),
+        );
+
         my $graceful_stop_o = rx_from_event(Mojo::IOLoop->singleton, 'finish');
 
         # on graceful stop, boot all clients
@@ -227,7 +240,6 @@ sub register ($self, $app, $config) {
         my @answers = $sub->($multi_mode ? \@clone_states : $clone_states[0]);
         @answers = ([@answers]) if not $multi_mode;
 
-
         my $ea = each_array(@rows, @answers, @orig_states);
         ANSWER:
         while( my ($row, $answer, $orig_state) = $ea->() ) {
@@ -236,6 +248,8 @@ sub register ($self, $app, $config) {
             my ($channel_id, $channel_name) = $row->@{qw/ id name /};
             next ANSWER unless defined $event or defined $new_state;
             my $diff = defined $new_state ? diff($orig_state, $new_state, noO => 1, noU => 1) : undef;
+            $diff = undef if $diff and ! %$diff;
+            next ANSWER unless defined $event or defined $diff;
             my ($event_id, $dt);
             if (defined $event) {
                 ($event_id, $dt) = $db->insert('event_patch',
@@ -296,13 +310,29 @@ sub register ($self, $app, $config) {
                 }
             }
 
-            $db->notify($channel_name => encode_json {
-                id    => int $event_id,
-                data  => {
-                    event => $event,
-                    patch => $diff,
-                },
-            });
+            {
+                my $notification = encode_json({
+                    id   => int $event_id,
+                    data => {
+                        event => $event,
+                        patch => $diff,
+                    },
+                });
+
+                if (length $notification > $notify_payload_size_limit) {
+                    my $i = 0;
+                    while (my $len = length $notification) {
+                        my $prefix =
+                            $len > $notify_payload_size_limit ? $i++ :
+                            'end';
+                        substr($notification, 0, 0, ":$prefix: ");
+
+                        $db->notify($channel_name, substr($notification, 0, $notify_payload_size_limit, ''));
+                    }
+                } else {
+                    $db->notify($channel_name, $notification);
+                }
+            }
         }
 
         $tx->commit unless $opts{no_txn};
@@ -312,13 +342,6 @@ sub register ($self, $app, $config) {
         return $multi_mode ? \@new_states : $new_states[0];
     });
 
-    # ping every 15 seconds
-    my $ping_o = rx_defer(sub {
-        rx_timer(rand(15), 15);
-    })->pipe(
-        op_share(),
-    );
-
     $app->helper("$prefix.on_join", sub ($c, $channel_name, $join_sub) {
         BoardStreams::Registry->add_join($channel_name, $join_sub);
     });
